﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace GraphicsDemo
{
    public partial class Form1 : Form
    {
        private List<GraphicsElement> elements = new List<GraphicsElement>();
        private GraphicsElement current = null;
        private bool draw = false;

        public Form1()
        {
            InitializeComponent();
            pictureBox1.Image = new Bitmap(pictureBox1.Width, pictureBox1.Height);
        }

        private void novyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            using(Graphics g = Graphics.FromImage(pictureBox1.Image))
            {
                g.Clear(pictureBox1.BackColor);
            }
            elements = new List<GraphicsElement>();
        }

        private void pictureBox1_MouseMove(object sender, MouseEventArgs e)
        {
            xStatusLabel.Text = e.X.ToString();
            yStatusLabel.Text = e.Y.ToString();
            if(draw)
            {
                clearCurrent();
                current.DX = e.X - current.X;
                current.DY = e.Y - current.Y;
                drawCurrent();

            }
        }

        private void drawCurrent()
        {
            using (Graphics g = Graphics.FromImage(pictureBox1.Image))
            {
                Pen pen = Pens.Black;
                switch (current.type)
                {
                    case ElementType.ELLIPSE:
                        drawEllipse(g, current, pen);
                        break;
                    case ElementType.LINE:
                        drawLine(g, current, pen);
                        break;
                    case ElementType.RECTANGLE:
                        drawRectangle(g, current, pen);
                        break;
                }
            }
            Refresh();
        }

        private void clearCurrent()
        {
            using (Graphics g = Graphics.FromImage(pictureBox1.Image))
            {
                Pen pen = new Pen(pictureBox1.BackColor);
                switch (current.type)
                {
                    case ElementType.ELLIPSE:
                        drawEllipse(g, current, pen);
                        break;
                    case ElementType.LINE:
                        drawLine(g, current, pen);
                        break;
                    case ElementType.RECTANGLE:
                        drawRectangle(g, current, pen);
                        break;
                }
            }
            Refresh();
        }

        private void drawRectangle(Graphics g, GraphicsElement current, Pen pen)
        {
            int x = current.X, y = current.Y, dx = current.DX, dy = current.DY;
            if (dx < 0)
            {
                x = x + dx;
                dx = -dx;
            }
            if(dy < 0)
            {
                y = y + dy;
                dy = -dy;
            }
            g.DrawRectangle(pen, x, y, dx, dy);
        }

        private void drawLine(Graphics g, GraphicsElement current, Pen pen)
        {
            g.DrawLine(pen, current.X, current.Y, current.X + current.DX, current.Y + current.DY);
        }

        private void drawEllipse(Graphics g, GraphicsElement current, Pen pen)
        {
            int x = current.X, y=current.Y, dx=current.DX, dy=current.DY;
            if(current.DX < 0)
            {
                x = current.X + current.DX;
                dx = -current.DX;
            }
            if(current.DY < 0)
            {
                y = current.Y + current.DY;
                dy = -current.DY;
            }
            g.DrawEllipse(pen, x, y, dx, dy);
        }

        private void ulozitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            pictureBox1.Image.Save("h:\\picture.bmp");
        }

        private void konecToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void caraToolStripMenuItem_Click(object sender, EventArgs e)
        {
            clearChecked();
            caraToolStripMenuItem.Checked = true;
            createCurrent(ElementType.LINE);
        }

        private void clearChecked()
        {
            caraToolStripMenuItem.Checked = false;
            obdelnikToolStripMenuItem.Checked = false;
            elipsaToolStripMenuItem.Checked = false;
        }

        private void createCurrent(ElementType type)
        {
            current = new GraphicsElement();
            current.type = type;
        }

        private void elipsaToolStripMenuItem_Click(object sender, EventArgs e)
        {
            clearChecked();
            elipsaToolStripMenuItem.Checked = true;
            createCurrent(ElementType.ELLIPSE);
        }

        private void obdelnikToolStripMenuItem_Click(object sender, EventArgs e)
        {
            clearChecked();
            obdelnikToolStripMenuItem.Checked = true;
            createCurrent(ElementType.RECTANGLE);
        }

        private void pictureBox1_MouseDown(object sender, MouseEventArgs e)
        {
            draw = true;
            if(current != null)
            {
                current.X = e.X;
                current.Y = e.Y;
            }
        }

        private void pictureBox1_MouseUp(object sender, MouseEventArgs e)
        {
            elements.Add(current);
            current = null;
            refreshElements();
            draw = false;
            pictureBox1.Refresh();
        }

        private void refreshElements()
        {
            using (Graphics g = Graphics.FromImage(pictureBox1.Image))
            {
                g.Clear(pictureBox1.BackColor);
                foreach (GraphicsElement element in elements)
                {
                    Pen pen = Pens.Black;
                    switch (element.type)
                    {
                        case ElementType.ELLIPSE:
                            drawEllipse(g, element, pen);
                            break;
                        case ElementType.LINE:
                            drawLine(g, element, pen);
                            break;
                        case ElementType.RECTANGLE:
                            drawRectangle(g, element, pen);
                            break;
                    }
                }
            }
        }
    }
}
